# collapsing_appbar_with_tabs

This flutter package is a wrapper around using `NestedScrollView`, `SliverAppBar` with `TabBar`, `SliverOverlapAbsorber`, `SliverOverlapInjector`, and `TabBarView` in order to have Collapsing Toolbar with Tabs.

## Motivation :

When I tried to make a Collaping toolbar with tabs using `SliverAppBar`, `SliverFillRemaining`, and `TabBarView`, I got some unwanted behaviors like extra scroll area and some overlapping, that's when I came upon [this example](https://api.flutter.dev/flutter/widgets/NestedScrollView-class.html#widgets.NestedScrollView.1) in flutter docs but it was complex and would be hard to maintain it in every project, so I created this simple wrapper to save some time and sanity.

### Screenshots :

_Nothing special_

![expanded](screenshots/1.jpg) ![collapesd](screenshots/2.jpg)

## Usage :

- Add to dependencies :
  ```yaml
  dependencies:
    collapsing_appbar_with_tabs: ^0.0.3
  ```
- Use with a `DefaultTabController` :

  ```dart
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: DefaultTabController(
        length: 3,
        child: CollapsingAppBarWithTabs(
          // appBar must be a SliverAppBar
          appBar: SliverAppBar(
            floating: true,
            pinned: true,
            expandedHeight: 200,
            bottom: TabBar(
              tabs: [
                Tab(text: "Tab one"),
                Tab(text: "Tab two"),
                Tab(text: "Tab three"),
              ],
            ),
          ),
          // tabs must be slivers like `SliverList`, `SliverFixedExtentList`,
          // or `SliverToBoxAdapter`
          tabs: [
            SliverFixedExtentList(
              itemExtent: 50.0,
              delegate: SliverChildBuilderDelegate(
                (BuildContext context, int index) {
                  return Container(
                    alignment: Alignment.center,
                    color: Colors.lightBlue[100 * (index % 9)],
                    child: Text('list item $index'),
                  );
                },
              ),
            ),
            SliverToBoxAdapter(
              child: Text("Tab two"),
            ),
            SliverToBoxAdapter(
              child: Text("Tab three"),
            ),
          ],
        ),
      ),
    );
  }
  ```

- Or use with a custom `TabController`

  ```dart
  TabController _tabController;

  @override
  void initState() {
    super.initState();
    _tabController = TabController(length: 3, vsync: this);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: CollapsingAppBarWithTabs(
        // appBar must be a SliverAppBar
        appBar: SliverAppBar(
          floating: true,
          pinned: true,
          expandedHeight: 200,
          bottom: TabBar(
            controller: _tabController,
            tabs: [
              Tab(text: "Tab one"),
              Tab(text: "Tab two"),
              Tab(text: "Tab three"),
            ],
          ),
        ),
        tabsController: _tabController,
        // tabs must be slivers like `SliverList`
        // or `SliverToBoxAdapter`
        tabs: [
          SliverToBoxAdapter(
            child: Text("Tab one"),
          ),
          SliverToBoxAdapter(
            child: Text("Tab two"),
          ),
          SliverToBoxAdapter(
            child: Text("Tab three"),
          ),
        ],
      ),
    );
  }
  ```
