library collapsing_appbar_with_tabs;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// This widget is just a wrapper around using [NestedScrollView] with
/// a [SliverOverlapAbsorber] and [SliverOverlapInjector] in order to get
/// [SliverAppBar] to work with [TabBarView] without any weird extra scroll or overlapping.
class CollapsingAppBarWithTabs extends StatelessWidget {
  /// The `appBar` should be the [SliverAppBar] that contains a tabbar.
  final Widget appBar;

  /// The `tabs` are a list of tabs be used inside the [TabBarView]  included with this class.
  final List<Widget> tabs;

  /// The `tabsController` is to be used to control the [TabBarView] included with this class.
  final TabController tabsController;

  CollapsingAppBarWithTabs(
      {Key key, this.appBar, this.tabs, this.tabsController})
      : assert(appBar is SliverAppBar),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return NestedScrollView(
      headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
        return [
          SliverOverlapAbsorber(
            handle: NestedScrollView.sliverOverlapAbsorberHandleFor(context),
            sliver: appBar,
          )
        ];
      },
      body: TabBarView(
        controller: tabsController,
        children: tabs
            .asMap()
            .entries
            .map((tab) => _TabViewWrapper(
                  child: tab.value,
                  storageKey: tab.key.toString(),
                ))
            .toList(),
      ),
    );
  }
}

class _TabViewWrapper extends StatelessWidget {
  /// The `child` should be of a [Sliver] like [SliverList] or [SliverToBoxAdapter].
  final Widget child;

  /// the `storageKey` is to keep track of each Tab scrolling position.
  final String storageKey;

  const _TabViewWrapper({Key key, this.child, this.storageKey})
      : assert(storageKey is String),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return CustomScrollView(
      key: PageStorageKey<String>(storageKey),
      slivers: [
        SliverOverlapInjector(
          handle: NestedScrollView.sliverOverlapAbsorberHandleFor(context),
        ),
        child,
      ],
    );
  }
}
